class UsefulConfig:
    COF_FUND_ID = "47aef2f5-3fcb-4d45-acb5-f0152b5f03c4"
    COF_ROUND_2_ID = "c603d114-5364-4474-a0c4-c41cbf4d3bbd"
    COF_ROUND_2_W3_ID = "5cf439bf-ef6f-431e-92c5-a1d90a4dd32f"



import os

from flask import current_app


class Healthcheck(object):
    def __init__(self, app):
        self.flask_app = app
        self.flask_app.add_url_rule("/healthcheck", view_func=self.healthcheck_view)
        self.checkers = []

    def healthcheck_view(self):
        responseCode = 200
        response = {"checks": []}
        version = os.getenv("GITHUB_SHA")
        if version:
            response["version"] = version
        for checker in self.checkers:
            try:
                result = checker.check()
                current_app.logger.debug(f"Check {checker.name} returned {result}")
                response["checks"].append({checker.name: result[1]})
                if result[0] is False:
                    responseCode = 500
            except Exception:
                response["checks"].append({checker.name: "Failed - check logs"})
                current_app.logger.exception(
                    f"Check {checker.name} failed with an exception"
                )
                responseCode = 500
        return response, responseCode

    def add_check(self, checker):
        self.checkers.append(checker)



import pytest
from fsd_utils.mapping.application.application_utils import convert_bool_value
from fsd_utils.mapping.application.application_utils import format_answer
from fsd_utils.mapping.application.application_utils import format_checkbox
from fsd_utils.mapping.application.application_utils import simplify_title
from fsd_utils.mapping.application.free_text import FreeText
from fsd_utils.mapping.application.multi_input import MultiInput
from fsd_utils.mapping.application.qa_mapping import extract_questions_and_answers
from tests.test_data_utils import multi_input_test_data
from tests.test_data_utils import test_data_sort_questions_answers


@pytest.mark.parametrize(
    "input_data, expected_response",
    [
        (True, "Yes"),
        (False, "No"),
        (["address", True], ["address", "Yes"]),
        ((["address", False], ["address", "No"])),
        (None, "Not provided"),
        ([["name", True], ["name", "Yes"]]),
        ([["name", False], ["name", "No"]]),
    ],
)
def test_convert_bool_values(input_data, expected_response):
    response = convert_bool_value(input_data)
    assert response == expected_response


@pytest.mark.parametrize(
    "input_data, expected_response",
    [
        ("null", ""),
        (None, "Not provided"),
    ],
)
def test_format_answer(input_data, expected_response):
    response = format_answer(input_data)
    assert response == expected_response


@pytest.mark.parametrize(
    "section_name, remove_text, expected_response",
    [
        (
            "organisation-type-cof-round",
            ["cof", "ns"],
            ["organisation", "type"],
        ),
        (
            "organisation-name-ns-round",
            ["cof", "ns"],
            ["organisation", "name"],
        ),
        ("organisation-address", ["cof", "ns"], ["organisation", "address"]),
    ],
)
def test_simplify_title(section_name, remove_text, expected_response):
    response = simplify_title(section_name, remove_text)
    assert response == expected_response


@pytest.mark.parametrize(
    "input_value, expected_response",
    [
        ("<p> Hello world </p>", "Hello world"),
        ("<ul><li>Item 1</li><li>Item 2</li></ul>", ". Item 1\n     . Item 2"),
        (
            "<ol><li>Item 1</li><li>Item 2</li></ol>",
            "1. Item 1\n     2. Item 2",
        ),
        ("text without html tags", "text without html tags"),
        (None, None),
        (["one", "two"], ["one", "two"]),
        (True, True),
        ("https://my-website.com", "https://my-website.com"),
    ],
)
def test_remove_html_tags(input_value, expected_response):

    response = FreeText.remove_html_tags(input_value)
    assert response == expected_response


def test_sort_questions_and_answers(app_context):
    forms = test_data_sort_questions_answers["forms"]

    response = extract_questions_and_answers(forms)

    assert response == test_data_sort_questions_answers["questions_answers"]


def test_sort_questions_and_answers_fail(app_context):
    forms = test_data_sort_questions_answers["incorrect_form_data"]

    with pytest.raises(Exception) as exc:
        extract_questions_and_answers(forms)

    assert str(exc.value) == test_data_sort_questions_answers["exception_message"]


@pytest.mark.parametrize(
    "input_value, expected_response",
    [
        (
            [
                "health-interventions",
                "employment-support",
            ],
            "- health interventions\n     - employment support",
        ),
        (
            ["Survivors of domestic abuse", "ethnic minorities"],
            "- Survivors of domestic abuse\n     - ethnic minorities",
        ),
    ],
)
def test_format_checkbox(input_value, expected_response):

    response = format_checkbox(input_value)
    assert response == expected_response


class TestMultiInput:
    @pytest.mark.parametrize(
        "input_data, expected_response",
        [
            (
                multi_input_test_data["process_data"]["multiple_values"]["input_data"],
                multi_input_test_data["process_data"]["multiple_values"][
                    "expected_response"
                ],
            ),
            (
                multi_input_test_data["process_data"]["single_value"]["input_data"],
                multi_input_test_data["process_data"]["single_value"][
                    "expected_response"
                ],
            ),
            (
                multi_input_test_data["process_data"]["iso_values"]["input_data"],
                multi_input_test_data["process_data"]["iso_values"][
                    "expected_response"
                ],
            ),
        ],
    )
    def test_process_data(self, app_context, input_data, expected_response):

        response = MultiInput.process_data(input_data)

        assert response == expected_response

    @pytest.mark.parametrize(
        "input_data, expected_response",
        [
            (
                multi_input_test_data["map_data"]["multiple_values"]["input_data"],
                multi_input_test_data["map_data"]["multiple_values"][
                    "expected_response"
                ],
            ),
            (
                multi_input_test_data["map_data"]["single_value"]["input_data"],
                multi_input_test_data["map_data"]["single_value"]["expected_response"],
            ),
            (
                multi_input_test_data["map_data"]["integer_values"]["input_data"],
                multi_input_test_data["map_data"]["integer_values"][
                    "expected_response"
                ],
            ),
            (
                multi_input_test_data["map_data"]["nested_dict_value_with_str_value"][
                    "input_data"
                ],
                multi_input_test_data["map_data"]["nested_dict_value_with_str_value"][
                    "expected_response"
                ],
            ),
        ],
    )
    def test_map_multi_input_data(self, app_context, input_data, expected_response):

        response = MultiInput.map_multi_input_data(input_data)

        assert response == expected_response



import pytest
from fsd_utils.simple_utils.data_utils import get_remote_data_as_json
from requests.exceptions import HTTPError


class TestDataUtils:
    def test_get_remote_data_json(self, flask_test_client):
        postcode = "SW1P 4DF"
        result = get_remote_data_as_json(f"https://postcodes.io/postcodes/{postcode}")
        assert postcode == result["result"]["postcode"]

    def test_get_remote_data_json_404(self, flask_test_client):
        with pytest.raises(HTTPError):
            get_remote_data_as_json("https://postcodes.io/postcodes/QQ11QQ")


multi_input_test_data = {
    "process_data": {
        "multiple_values": {
            "input_data": {
                "trusts one": [
                    125,
                    "1 April 2023 to 31 March 2024",
                    "Capital",
                    True,
                ],
                "trust two": [
                    456,
                    "1 April 2024 to 31 March 2025",
                    "Revenue",
                    False,
                ],
            },
            "expected_response": [
                "- trusts one: [125, '1 April 2023 to 31 March 2024',"
                " 'Capital', 'Yes']",
                "     - trust two: [456, '1 April 2024 to 31 March 2025',"
                " 'Revenue', 'No']",
            ],
        },
        "single_value": {
            "input_data": {
                "bbd0ec2a-972f-4d06-bf93-bf24786c3859": "Sky builders",
                "ac8bbdfe-6a39-45b8-8c0a-6558148388d1": "trust builders",
            },
            "expected_response": ["- Sky builders", "     - trust builders"],
        },
        "iso_values": {
            "input_data": {
                "Project one": [{"PrulfI__month": 1, "PrulfI__year": 2021}],
                "Project two": [{"PrulfI__month": 2, "PrulfI__year": 2022}],
            },
            "expected_response": (
                [
                    "- Project one: ['month: 1', 'year: 2021']",
                    "     - Project two: ['month: 2', 'year: 2022']",
                ]
            ),
        },
    },
    "map_data": {
        "multiple_values": {
            "input_data": [
                {
                    "AfAKxk": "trusts one",
                    "CrcLtW": 125,
                    "ndySbC": "1 April 2023 to 31 March 2024",
                    "pATWyM": "Capital",
                    "sIFBGc": True,
                },
                {
                    "AfAKxk": "trust two",
                    "CrcLtW": 456,
                    "ndySbC": "1 April 2024 to 31 March 2025",
                    "pATWyM": "Revenue",
                    "sIFBGc": False,
                },
            ],
            "expected_response": (
                "- trusts one: [125, '1 April 2023 to 31 March 2024',"
                " 'Capital', 'Yes']\n     - trust two: [456, '1 April 2024 to"
                " 31 March 2025', 'Revenue', 'No']"
            ),
        },
        "single_value": {
            "input_data": [
                {"CZZYvE": "Sky builders"},
                {"CZZYvE": "trust builders"},
            ],
            "expected_response": "- Sky builders\n     - trust builders",
        },
        "integer_values": {
            "input_data": [
                {"GLQlOh": "cost one", "JtwkMy": 4444},
                {"GLQl6y": "cost two", "JtwkMt": 4455},
            ],
            "expected_response": "- cost one: 4444\n     - cost two: 4455",
        },
        "nested_dict_value_with_str_value": {
            "input_data": [
                {
                    "PrulfI": {"PrulfI__month": 2, "PrulfI__year": 2022},
                    "fFIuPP": "Milestone one",
                },
                {
                    "fFIuPP": "Milestone two",
                    "PrulfI": {"PrulfI__month": 3, "PrulfI__year": 2023},
                },
            ],
            "expected_response": (
                "- Milestone one: ['month: 2', 'year: 2022']\n     - Milestone"
                " two: ['month: 3', 'year: 2023']"
            ),
        },
    },
}


test_data_sort_questions_answers = {
    "forms": [
        {
            "questions": [
                {
                    "category": "ZbxIUV",
                    "question": "Lead contact details",
                    "fields": [
                        {
                            "key": "fUMWcd",
                            "title": "Name of lead contact",
                            "type": "text",
                            "answer": "test name",
                        },
                        {
                            "key": "ayzqnK",
                            "title": (
                                "Is the lead contact the same person as the"
                                " authorised signatory?"
                            ),
                            "type": "list",
                            "answer": True,
                        },
                    ],
                }
            ],
            "name": "applicant-information-ns",
        },
    ],
    "form_names": [
        "objectives-and-activities-ns",
        "risk-and-deliverability-ns",
        "applicant-information-ns",
    ],
    "questions_answers": {
        "applicant-information-ns": {
            "Name of lead contact": "test name",
            "Is the lead contact the same person as the authorised signatory?": ("Yes"),
        }
    },
    "incorrect_form_data": [
        {
            "questions": [
                {
                    "category": "ZbxIUV",
                    "question": "Lead contact details",
                    "fields": [
                        {
                            "key": "fUMWcd",
                            "type": "text",
                            "answer": "test name",
                        },
                    ],
                }
            ],
            "name": "applicant-information-ns",
        },
    ],
    "exception_message": ("Could not map the data for form: applicant-information-ns"),
}



from pathlib import Path

import jwt as jwt


class TestAuthentication:

    test_payload = {
        "accountId": "test-user",
        "email": "test@example.com",
        "fullName": "Test User",
        "roles": ["COF_LEAD_ASSESSOR", "COF_ASSESSOR", "COF_COMMENTER"],
    }
    expected_valid_g_attributes = {
        "is_authenticated": True,
        "logout_url": "https://authenticator/sessions/sign-out",
        "account_id": "test-user",
        "user": {
            "email": "test@example.com",
            "full_name": "Test User",
            "highest_role_map": {"COF": "LEAD_ASSESSOR"},
            "roles": ["COF_LEAD_ASSESSOR", "COF_ASSESSOR", "COF_COMMENTER"],
        },
    }

    expected_unauthenticated_g_attributes = {
        "is_authenticated": False,
        "logout_url": "https://authenticator/sessions/sign-out",
        "account_id": None,
    }

    def _create_valid_token(self):

        _test_private_key_path = str(Path(__file__).parent) + "/keys/rsa256/private.pem"
        with open(_test_private_key_path, mode="rb") as private_key_file:
            rsa256_private_key = private_key_file.read()

            return jwt.encode(self.test_payload, rsa256_private_key, algorithm="RS256")

    def _create_invalid_token(self):

        _test_private_key_path = (
            str(Path(__file__).parent) + "/keys/rsa256/private_invalid.pem"
        )
        with open(_test_private_key_path, mode="rb") as private_key_file:
            rsa256_private_key = private_key_file.read()

            return jwt.encode(self.test_payload, rsa256_private_key, algorithm="RS256")

    def test_login_required_redirects_to_signed_out_without_token(
        self, flask_test_client
    ):
        """
        GIVEN a flask_test_client and
            route decorated with @login_required decorator
        WHEN a request is made without any "fsd-user-token" cookie
        THEN the route redirects to the authenticator /sessions/sign-out url
        :param flask_test_client:
        """
        mock_request = flask_test_client.get("/mock_login_required_route")

        assert mock_request.status_code == 302
        assert mock_request.location == "https://authenticator/sessions/sign-out"

    def test_login_required_redirects_to_signed_out_with_invalid_token(
        self, flask_test_client
    ):
        """
        GIVEN a flask_test_client and
            route decorated with @login_required decorator
        WHEN a request is made with a correctly formatted
            but invalidly signed "fsd-user-token" cookie
        THEN the route redirects to the authenticator /sessions/sign-out url
        :param flask_test_client:
        """
        invalid_token = self._create_invalid_token()
        flask_test_client.set_cookie("localhost", "fsd-user-token", invalid_token)
        mock_request = flask_test_client.get("/mock_login_required_route")

        assert mock_request.status_code == 302
        assert mock_request.location == "https://authenticator/sessions/sign-out"

    def test_login_required_sets_user_attributes_with_valid_token(
        self, flask_test_client
    ):
        """
        GIVEN a flask_test_client and
            route decorated with @login_required decorator
        WHEN a request is made with a correctly formatted
            and signed "fsd-user-token" cookie
        THEN the route returns with the correct flask g variables set
        :param flask_test_client:
        """
        valid_token = self._create_valid_token()
        flask_test_client.set_cookie("localhost", "fsd-user-token", valid_token)
        mock_request = flask_test_client.get("/mock_login_required_route")
        assert mock_request.status_code == 200
        assert mock_request.json == self.expected_valid_g_attributes

    def test_login_required_roles_redirects_to_error_if_missing_roles(
        self, flask_test_client
    ):
        """
        GIVEN a flask_test_client and route decorated with
            @login_required(roles_required=["ADMIN"]) decorator
        WHEN a request is made with a correctly formatted
            and signed "fsd-user-token" cookie
            but without "ADMIN" in the roles param
        THEN the route redirects to the authenticator /sessions/sign-out url
        :param flask_test_client:
        """
        valid_token = self._create_valid_token()
        flask_test_client.set_cookie("localhost", "fsd-user-token", valid_token)
        mock_request = flask_test_client.get("/mock_login_required_admin_roles_route")
        assert mock_request.status_code == 302
        assert (
            mock_request.location
            == "https://authenticator/service/user?roles_required=COF_ADMIN|COF_TEST"
        )

    def test_login_required_roles_sets_user_attributes_if_user_has_roles(
        self, flask_test_client
    ):
        """
        GIVEN a flask_test_client and route decorated with
            @login_required(roles_required=["COMMENTER"]) decorator
        WHEN a request is made with a correctly formatted
            and signed "fsd-user-token" cookie
        THEN the route returns with the correct flask g variables set
        :param flask_test_client:
        """
        valid_token = self._create_valid_token()
        flask_test_client.set_cookie("localhost", "fsd-user-token", valid_token)
        mock_request = flask_test_client.get("/mock_login_required_roles_route")
        assert mock_request.status_code == 200
        assert mock_request.json == self.expected_valid_g_attributes

    def test_login_requested_sets_is_authenticated_to_false_with_no_token(
        self, flask_test_client
    ):
        """
        GIVEN a flask_test_client and
            route decorated with @login_requested decorator
        WHEN a request is without any "fsd-user-token" cookie
        THEN the route returns with flask g.is_authenticated == False
        :param flask_test_client:
        """
        mock_request = flask_test_client.get("/mock_login_requested_route")
        assert mock_request.status_code == 200
        assert mock_request.json == self.expected_unauthenticated_g_attributes

    def test_login_requested_sets_user_attributes_with_valid_token(
        self, flask_test_client
    ):
        """
        GIVEN a flask_test_client and
            route decorated with @login_requested decorator
        WHEN a request is made with a correctly formatted
            and signed "fsd-user-token" cookie
        THEN the route returns with the correct flask g variables set
        :param flask_test_client:
        """
        valid_token = self._create_valid_token()
        flask_test_client.set_cookie("localhost", "fsd-user-token", valid_token)
        mock_request = flask_test_client.get("/mock_login_requested_route")
        assert mock_request.status_code == 200
        assert mock_request.json == self.expected_valid_g_attributes

    def test_login_required_roles_debuggable_in_development(
        self, flask_test_development_client
    ):
        """
        GIVEN a flask_test_client and route decorated with
            @login_required(roles_required=["ADMIN"]) decorator
            and env vars of:
                FLASK_ENV = 'development'
                DEBUG_USER_ROLE = "ADMIN"
        WHEN a request is made without a valid fsd_user_token cookie
        THEN the route is still accessible
        :param flask_test_client:
        """
        flask_test_development_client.set_cookie("localhost", "fsd-user-token", "")
        mock_request = flask_test_development_client.get(
            "/mock_login_required_admin_roles_route"
        )
        assert mock_request.status_code == 200

    def test_login_required_roles_debuggable_but_still_requires_roles(
        self, flask_test_development_client
    ):
        """
        GIVEN a flask_test_client and route decorated with
            @login_required(roles_required=["COMMENTER"]) decorator
            and env vars of:
                FLASK_ENV = 'development'
                DEBUG_USER_ROLE = "ADMIN"
        WHEN a request is made without a valid fsd_user_token cookie
        THEN the route redirects to the authenticator /sessions/sign-out url
        :param flask_test_client:
        """
        flask_test_development_client.set_cookie("localhost", "fsd-user-token", "")
        mock_request = flask_test_development_client.get(
            "/mock_login_required_roles_route"
        )
        assert mock_request.status_code == 302
        assert (
            mock_request.location
            == "https://authenticator/service/user?roles_required=COF_COMMENTER"
        )

    def test_login_required_with_return_app_redirects_to_signed_out_without_token(
        self, flask_test_client
    ):
        """
        GIVEN a flask_test_client and
            route decorated with @login_required decorator with the "return_app" parameter set to "post-award-frontend"
        WHEN a request is made without any "fsd-user-token" cookie
        THEN the route redirects to the authenticator /sessions/sign-out url with correct return_app query param
        :param flask_test_client:
        """
        mock_request = flask_test_client.get("/mock_login_requested_return_app_route")

        assert mock_request.status_code == 302
        assert (
            mock_request.location
            == "https://authenticator/sessions/sign-out?return_app=post-award-frontend"
        )

    def test_login_required_with_return_app_redirects_to_signed_out_with_invalid_token(
        self, flask_test_client
    ):
        """
        GIVEN a flask_test_client and
            route decorated with @login_required decorator with the "return_app" parameter set to "post-award-frontend"
        WHEN a request is made with a correctly formatted
            but invalidly signed "fsd-user-token" cookie
        THEN the route redirects to the authenticator /sessions/sign-out url with correct return_app query param
        :param flask_test_client:
        """
        invalid_token = self._create_invalid_token()
        flask_test_client.set_cookie("localhost", "fsd-user-token", invalid_token)
        mock_request = flask_test_client.get("/mock_login_requested_return_app_route")

        assert mock_request.status_code == 302
        assert (
            mock_request.location
            == "https://authenticator/sessions/sign-out?return_app=post-award-frontend"
        )

    def test_login_required_with_return_app_sets_user_attributes_with_valid_token(
        self, flask_test_client
    ):
        """
        GIVEN a flask_test_client and
            route decorated with @login_required decorator with the "return_app" parameter set to "post-award-frontend"
        WHEN a request is made with a correctly formatted
            and signed "fsd-user-token" cookie
        THEN the route returns with the g variable "logout_url" set correctly
        :param flask_test_client:
        """
        valid_token = self._create_valid_token()
        flask_test_client.set_cookie("localhost", "fsd-user-token", valid_token)
        mock_request = flask_test_client.get("/mock_login_requested_return_app_route")
        assert mock_request.status_code == 200
        assert (
            mock_request.json["logout_url"]
            == "https://authenticator/sessions/sign-out?return_app=post-award-frontend"
        )
